==> atlassian_connect_jwt-0.3.2.tar.gz: test/qsh_test.dart <==
library ac_dart.ac_jwt.qsh.test;

import 'package:atlassian_connect_jwt/src/qsh.dart';
import 'package:test/test.dart';
import 'package:quiver/collection.dart';

void main() {
  String _canonicalUriStr(String uri, String contextPath,
          [String method = 'GET']) =>
      new CanonicalUri(Uri.parse(uri), method, contextPath).canonicalUri;

  group('[canonicalisation]', () {
    group('[preconditions]', () {
      test('throws ArgumentError when uri null', () {
        expect(() => new CanonicalUri(null, 'GET', null),
            throwsA(isArgumentError));
      });

      test('throws ArgumentError when uri null', () {
        expect(() => new CanonicalUri(
                Uri.parse("http://localhost:2990/path/to/service"), null, null),
            throwsA(isArgumentError));
      });
    });

    group('[path]', () {
      runPathTests(String canonicalUriStr(String contextPath)) {
        test('full path included when no context path', () {
          expect(canonicalUriStr(null), equals("GET&/path/to/service&"));
        });

        test('full path included when / context path', () {
          expect(canonicalUriStr('/'), equals("GET&/path/to/service&"));
        });

        test('full path included when empty context path', () {
          expect(canonicalUriStr(''), equals("GET&/path/to/service&"));
        });

        test('full path included when // context path', () {
          expect(canonicalUriStr('//'), equals("GET&/path/to/service&"));
        });

        test('removes context path when provided', () {
          expect(canonicalUriStr('path'), equals("GET&/to/service&"));
        });

        test('removes context path when provided with /', () {
          expect(canonicalUriStr('/path/'), equals("GET&/to/service&"));
        });
      }

      group('[relative]', () {
        runPathTests((String contextPath) =>
            _canonicalUriStr("/path/to/service", contextPath));
      });

      group('[relative - no /]', () {
        runPathTests((String contextPath) =>
            _canonicalUriStr("path/to/service", contextPath));
      });

      group('[absolute]', () {
        runPathTests((String contextPath) => _canonicalUriStr(
            "http://localhost:2990/path/to/service", contextPath));
      });

      group('[no path]', () {
        test('path is slash if no path - context /', () {
          expect(
              _canonicalUriStr('http://localhost:2990', '/'), equals('GET&/&'));
        });

        test('path is slash if / path - context /', () {
          expect(_canonicalUriStr('http://localhost:2990/', '/'),
              equals('GET&/&'));
        });

        test('path is slash if no path - context null', () {
          expect(_canonicalUriStr('http://localhost:2990', null),
              equals('GET&/&'));
        });

        test('path is slash if no path - context null', () {
          expect(_canonicalUriStr('http://localhost:2990/', null),
              equals('GET&/&'));
        });

        test('path is slash if no path - context empty', () {
          expect(
              _canonicalUriStr('http://localhost:2990', ''), equals('GET&/&'));
        });

        test('path is slash if no path - context empty', () {
          expect(
              _canonicalUriStr('http://localhost:2990/', ''), equals('GET&/&'));
        });
      });
    });

    group('[params]', () {
      String canonicalUriStr(String params) => _canonicalUriStr(
          "http://localhost:2990/path/to/service?$params", null);

      test('single param is included as is', () {
        expect(canonicalUriStr("param=only"),
            equals("GET&/path/to/service&param=only"));
      });

      test('repeated param is included comma separated', () {
        expect(canonicalUriStr("repeated=first&repeated=second"),
            equals("GET&/path/to/service&repeated=first,second"));
      });

      test('repeated param is included comma separated alphabetically', () {
        expect(canonicalUriStr("repeated=zorro&repeated=aardvark"),
            equals("GET&/path/to/service&repeated=aardvark,zorro"));
      });

      test('params are ordered alphabetically', () {
        expect(canonicalUriStr("aardvark=last&Aardvark=first"),
            equals("GET&/path/to/service&Aardvark=first&aardvark=last"));
      });

      test('param values are uri encoded', () {
        expect(canonicalUriStr("param=parameter 1"),
            equals("GET&/path/to/service&param=parameter%201"));
      });

      test('param keys are uri encoded', () {
        expect(canonicalUriStr("param 1=parameter"),
            equals("GET&/path/to/service&param%201=parameter"));
      });

      test('param values are uri encoded - path form', () {
        expect(new CanonicalUri.fromPath('/path/to/service', 'GET',
                new Multimap()..add('param', 'parameter 1')).canonicalUri,
            equals("GET&/path/to/service&param=parameter%201"));
      });

      test('param keys are uri encoded - path form', () {
        expect(new CanonicalUri.fromPath('/path/to/service', 'GET',
                new Multimap()..add('param 1', 'parameter')).canonicalUri,
            equals("GET&/path/to/service&param%201=parameter"));
      });

      test('jwt param is ignored', () {
        expect(canonicalUriStr("param=only&jwt=foo"),
            equals("GET&/path/to/service&param=only"));
      });

      test('all together now', () {
        expect(canonicalUriStr(
            "zee_last=param&repeated=parameter 1&first=param&repeated=parameter 2"), equals(
            "GET&/path/to/service&first=param&repeated=parameter%201,parameter%202&zee_last=param"));
      });
    });
  });

  group('[qsh]', () {
    test('qsh works', () {
      final uriStr =
          "/hello-world?lic=none&tz=Australia%2FSydney&cp=%2Fjira&user_key=&loc=en-US&user_id=&xdm_e=http%3A%2F%2Fstorm%3A2990&xdm_c=channel-servlet-hello-world&xdm_p=1";
      final uri = Uri.parse(uriStr);
      final actualHash = createQsh(uri, "GET", null);
      final expectedHash =
          "8063ff4ca1e41df7bc90c8ab6d0f6207d491cf6dad7c66ea797b4614b71922e9";
      expect(actualHash, equals(expectedHash));
    });
  });

  // If the separator is not URL encoded then the following URLs have the same query-string-hash:
  //   https://djtest9.jira-dev.com/rest/api/2/project&a=b?x=y
  //   https://djtest9.jira-dev.com/rest/api/2/project?a=b&x=y
  group('[canonical request]', () {
    test('an ampersand is url-encoded', () {
      expect(_canonicalUriStr(
              'https://server:1234/context/path&a=b?x=y', '/context', 'DELETE'),
          equals('DELETE&/path%26a=b&x=y'));
    });
    test('multiple ampersands are url-encoded', () {
      expect(_canonicalUriStr('https://server:1234/context/path&a=b&c=d?x=y',
          '/context', 'DELETE'), equals('DELETE&/path%26a=b%26c=d&x=y'));
    });
    test('ampersands in path do not result in canonical request collisions',
        () {
      final uriPrefix = 'https://server:1234/context/path';
      final context = '/context';
      expect(_canonicalUriStr(uriPrefix + '&a=b?x=y', context),
          isNot(equals(_canonicalUriStr(uriPrefix + '?a=b&x=y', context))));
    });
  });
}

==> atlassian_connect_jwt-0.3.4.tar.gz: test/qsh_test.dart <==
library ac_dart.ac_jwt.qsh.test;

import 'package:atlassian_connect_jwt/src/qsh.dart';
import 'package:test/test.dart';
import 'package:quiver/collection.dart';

void main() {
  String _canonicalUriStr(String uri, String contextPath,
          [String method = 'GET']) =>
      new CanonicalUri(Uri.parse(uri), method, contextPath).canonicalUri;

  group('[canonicalisation]', () {
    group('[preconditions]', () {
      test('throws ArgumentError when uri null', () {
        expect(() => new CanonicalUri(null, 'GET', null),
            throwsA(isArgumentError));
      });

      test('throws ArgumentError when uri null', () {
        expect(
            () => new CanonicalUri(
                Uri.parse("http://localhost:2990/path/to/service"), null, null),
            throwsA(isArgumentError));
      });
    });

    group('[path]', () {
      runPathTests(String canonicalUriStr(String contextPath)) {
        test('full path included when no context path', () {
          expect(canonicalUriStr(null), equals("GET&/path/to/service&"));
        });

        test('full path included when / context path', () {
          expect(canonicalUriStr('/'), equals("GET&/path/to/service&"));
        });

        test('full path included when empty context path', () {
          expect(canonicalUriStr(''), equals("GET&/path/to/service&"));
        });

        test('full path included when // context path', () {
          expect(canonicalUriStr('//'), equals("GET&/path/to/service&"));
        });

        test('removes context path when provided', () {
          expect(canonicalUriStr('path'), equals("GET&/to/service&"));
        });

        test('removes context path when provided with /', () {
          expect(canonicalUriStr('/path/'), equals("GET&/to/service&"));
        });
      }

      group('[relative]', () {
        runPathTests((String contextPath) =>
            _canonicalUriStr("/path/to/service", contextPath));
      });

      group('[relative - no /]', () {
        runPathTests((String contextPath) =>
            _canonicalUriStr("path/to/service", contextPath));
      });

      group('[absolute]', () {
        runPathTests((String contextPath) => _canonicalUriStr(
            "http://localhost:2990/path/to/service", contextPath));
      });

      group('[no path]', () {
        test('path is slash if no path - context /', () {
          expect(
              _canonicalUriStr('http://localhost:2990', '/'), equals('GET&/&'));
        });

        test('path is slash if / path - context /', () {
          expect(_canonicalUriStr('http://localhost:2990/', '/'),
              equals('GET&/&'));
        });

        test('path is slash if no path - context null', () {
          expect(_canonicalUriStr('http://localhost:2990', null),
              equals('GET&/&'));
        });

        test('path is slash if no path - context null', () {
          expect(_canonicalUriStr('http://localhost:2990/', null),
              equals('GET&/&'));
        });

        test('path is slash if no path - context empty', () {
          expect(
              _canonicalUriStr('http://localhost:2990', ''), equals('GET&/&'));
        });

        test('path is slash if no path - context empty', () {
          expect(
              _canonicalUriStr('http://localhost:2990/', ''), equals('GET&/&'));
        });
      });
    });

    group('[params]', () {
      String canonicalUriStr(String params) => _canonicalUriStr(
          "http://localhost:2990/path/to/service?$params", null);

      test('single param is included as is', () {
        expect(canonicalUriStr("param=only"),
            equals("GET&/path/to/service&param=only"));
      });

      test('repeated param is included comma separated', () {
        expect(canonicalUriStr("repeated=first&repeated=second"),
            equals("GET&/path/to/service&repeated=first,second"));
      });

      test('repeated param is included comma separated alphabetically', () {
        expect(canonicalUriStr("repeated=zorro&repeated=aardvark"),
            equals("GET&/path/to/service&repeated=aardvark,zorro"));
      });

      test('params are ordered alphabetically', () {
        expect(canonicalUriStr("aardvark=last&Aardvark=first"),
            equals("GET&/path/to/service&Aardvark=first&aardvark=last"));
      });

      test('param values are uri encoded', () {
        expect(canonicalUriStr("param=parameter 1"),
            equals("GET&/path/to/service&param=parameter%201"));
      });

      test('param keys are uri encoded', () {
        expect(canonicalUriStr("param 1=parameter"),
            equals("GET&/path/to/service&param%201=parameter"));
      });

      test('param values are uri encoded - path form', () {
        expect(
            new CanonicalUri.fromPath('/path/to/service', 'GET',
                new Multimap()..add('param', 'parameter 1')).canonicalUri,
            equals("GET&/path/to/service&param=parameter%201"));
      });

      test('param keys are uri encoded - path form', () {
        expect(
            new CanonicalUri.fromPath('/path/to/service', 'GET',
                new Multimap()..add('param 1', 'parameter')).canonicalUri,
            equals("GET&/path/to/service&param%201=parameter"));
      });

      test('jwt param is ignored', () {
        expect(canonicalUriStr("param=only&jwt=foo"),
            equals("GET&/path/to/service&param=only"));
      });

      test('all together now', () {
        expect(
            canonicalUriStr(
                "zee_last=param&repeated=parameter 1&first=param&repeated=parameter 2"),
            equals(
                "GET&/path/to/service&first=param&repeated=parameter%201,parameter%202&zee_last=param"));
      });
    });
  });

  group('[qsh]', () {
    test('qsh works', () {
      final uriStr =
          "/hello-world?lic=none&tz=Australia%2FSydney&cp=%2Fjira&user_key=&loc=en-US&user_id=&xdm_e=http%3A%2F%2Fstorm%3A2990&xdm_c=channel-servlet-hello-world&xdm_p=1";
      final uri = Uri.parse(uriStr);
      final actualHash = createQsh(uri, "GET", null);
      final expectedHash =
          "8063ff4ca1e41df7bc90c8ab6d0f6207d491cf6dad7c66ea797b4614b71922e9";
      expect(actualHash, equals(expectedHash));
    });
  });

  // If the separator is not URL encoded then the following URLs have the same query-string-hash:
  //   https://djtest9.jira-dev.com/rest/api/2/project&a=b?x=y
  //   https://djtest9.jira-dev.com/rest/api/2/project?a=b&x=y
  group('[canonical request]', () {
    test('an ampersand is url-encoded', () {
      expect(
          _canonicalUriStr(
              'https://server:1234/context/path&a=b?x=y', '/context', 'DELETE'),
          equals('DELETE&/path%26a=b&x=y'));
    });
    test('multiple ampersands are url-encoded', () {
      expect(
          _canonicalUriStr('https://server:1234/context/path&a=b&c=d?x=y',
              '/context', 'DELETE'),
          equals('DELETE&/path%26a=b%26c=d&x=y'));
    });
    test(
        'ampersands in path do not result in canonical request collisions', () {
      final uriPrefix = 'https://server:1234/context/path';
      final context = '/context';
      expect(_canonicalUriStr(uriPrefix + '&a=b?x=y', context),
          isNot(equals(_canonicalUriStr(uriPrefix + '?a=b&x=y', context))));
    });
  });
}
